--[[

         Titan Panel [Item Count]
         50200-1.03.1beta

]]--

TitanItemCount = LibStub("AceAddon-3.0"):NewAddon("TitanItemCount", "AceConsole-3.0", 
   "AceComm-3.0", "AceEvent-3.0", "AceSerializer-3.0", "AceTimer-3.0", "AceHook-3.0");
TitanItemCount.version = "50200-1.03.1beta";
TitanItemCount.author = "tfetherston (Solage of Greymane)";

local menutext = "Titan Item Count |cff22ee55(" .. 
   GetAddOnMetadata("TitanItemCount", "Version") .. ")";
local buttonlabel = "Item Count: "
local ID = "ItmC"

local elapQoH, QoH, prevQoH = 0, 0.0, -2
local GetUnitQoH = 0 
local startVal = 0
local OrigItemToCount

TitanItemCount_UpdateInterval = 1.0;


--[[
-- ------------------[ Settings for the Addon Options Panel ]---------------
]]--

local options = {
	name = "Item Count",
	handler = TitanItemCount,
	type = "group",
	args = {
		info1 = {
			type		= "description",
			name		= "Version " .. TitanItemCount.version,
			order		= 0,
		},
		info2 = {
			type		= "description",
			name		= "Authors: " .. TitanItemCount.author,
			order		= 1,
		},
		header = {
			type		= "header",
			name		= "General Options",
			order		= 2,
		},
		ItemToCount = {
			type = 'input',
			name = "Item to Count",
			desc = "For which item do you want to show quantity on-hand?",
			get = function() return TitanItemCount.db.char.ItemToCount end,
			set = "SetItemToCount",
			width = full,
			order = 3,
		},
		space1 = {
		   type = "description",
		   name = "",
		   order = 4,
		},
		ShowItemName = {
		   type = 'toggle',
		   name = "Show Item Name",
		   desc = "Show the item name in the Titan button?",
		   get = function() return TitanGetVar(ID,"ShowItemName") end,
		   set = "SetShowItemName",
		   width = full,
		   order = 5,
		},
	}
};


--[[
-- ------------------[ Saved Variable Data Table ]---------------
]]--

local defaults = {
	char = {
		ItemToCount = "Elixir of Water Walking",
		ShowItemName = true,
	}
};


--[[
-- ------------------[ Addon Object Functions ]---------------
]]--

function TitanItemCount:OnInitialize()
	-- Load Options DB
	self.db = LibStub("AceDB-3.0"):New("TitanItemCountDB", defaults);

	-- Initialize Config Panels
	self:InitConfig();

   -- Register Events

	-- Delay welcome message.
	self:ScheduleTimer(function() 
			self:Print("Titan Item Count (" .. self.version .. ") is loaded.  " ..
			   "Currently tracking [" .. TitanItemCount.db.char.ItemToCount .. 
			   "]  Use /tic to change tracked item.")
		end, 4);

end

function TitanItemCount:OnEnable()
	self:Hook("ContainerFrameItemButton_OnModifiedClick", true)
end


function TitanItemCount:InitConfig()

	LibStub("AceConfig-3.0"):RegisterOptionsTable("Item Count", options);
	LibStub("AceConfigDialog-3.0"):AddToBlizOptions("Item Count");

end


function TitanItemCount:ticAnnounce(msg)
   self:Print(YELLOW_FONT_COLOR_CODE .. "now counting " .. msg)
end


-- AltClick Handling
function TitanItemCount:ContainerFrameItemButton_OnModifiedClick(...)
   if select(2, ...) == "LeftButton" and IsAltKeyDown() and 
   not IsControlKeyDown() and not IsShiftKeyDown() and not CursorHasItem() then

		bagID, slot = (...):GetParent():GetID(), (...):GetID()
      texture, itemCount, locked, quality, readable, lootable, itemLink = 
         GetContainerItemInfo(bagID, slot);
      -- no need to validate, we know it's in the bags
      TitanItemCount.db.char.ItemToCount = itemLink
      TitanItemCount:ticAnnounce(itemLink)
      startVal = 0

   end
end


--[[    Interface with AceConfig Options Panel   ]]--

-- ItemToCount set db var to value entered in options panel
function TitanItemCount:SetItemToCount(info, input)
   TitanItemCount:ticAnnounce(input)
   TitanItemCount.db.char.ItemToCount = input
end


function TitanItemCount:SetShowItemName(info, input)
   TitanSetVar(ID, "ShowItemName", input);
   TitanItemCount.db.char.ShowItemName = input;
   TitanPanelButton_UpdateButton(ID)
end


-- ItemToCount validation
function TitanItemCount:ValidateItemToCount(info, input)
   local tcount
   tcount = GetItemCount(input)
   if tcount < 1 then return "Must be an item you're currently carrying"; end
end



--[[
-- ------------------[ Titan Panel Button Event handlers ]---------------
]]--

function TitanPanelItmCButton_OnUpdate(this, elapsed)
   if elapsed == nil then elapsed = 2; end
   elapQoH = elapQoH + elapsed;
   if elapQoH < 1 then return end

   QoH = GetItemCount(TitanItemCount.db.char.ItemToCount)

   if QoH == prevQoH then return end
   prevQoH = QoH
   TitanPanelButton_UpdateButton(ID)
   elapQoH = 0;

end


function TitanPanelItmCButton_OnLoad(self)
   if not TitanPanelButton_UpdateButton then
      return;
   end

	self.registry = {
		id = ID,
		menuText = menutext,
		buttonTextFunction = "TitanPanelItmCButton_GetButtonText",
		tooltipTitle = GREEN_FONT_COLOR_CODE .. "Item Count",
		tooltipTextFunction = "TitanPanelItmCButton_GetTooltipText",
		frequency = 2,
		icon = "Interface\\AddOns\\TitanBag\\TitanBag",
		iconWidth = 16,
		category = "Information",
		savedVariables = {
			ShowIcon = 1,
			ShowLabelText = false,
			ShowItemName = false,
		},
	}

end


function TitanPanelItmCButton_OnShow(self)
	TitanPanelButton_OnShow(self);
end


function TitanPanelItmCButton_OnHide()
end


function TitanPanelItmCButton_OnClick(self, button)
   if (button == "LeftButton") then
      InterfaceOptionsFrame_OpenToCategory("Item Count")
   end
end


function TitanPanelItmCButton_GetButtonText()
	local QoHtext
	if not QoH then
	   QoH = 0
	end
	QoHtext = string.format(" %.0f ", QoH)
	if TitanGetVar(ID, "ShowItemName") then
	   QoHtext = QoHtext .. TitanItemCount.db.char.ItemToCount
	end
	return buttonlabel, QoHtext
end


function TitanPanelItmCButton_GetTooltipText()
	return "You have " .. string.format("%.0f", QoH) .. " of ".. 
	   TitanItemCount.db.char.ItemToCount .." in your inventory"
end


--[[
-- ------------------[ Titan Panel Functions ]---------------
]]--

local temp = {}
local function UIDDM_Add(text, func, checked, keepShown)
	temp.text, temp.func, temp.checked, temp.keepShownOnClick = text, 
	   func, checked, keepShown;
	UIDropDownMenu_AddButton(temp)
end

function TitanPanelRightClickMenu_PrepareItmCMenu()
	TitanPanelRightClickMenu_AddTitle(TitanPlugins[ID].menuText)	
	TitanPanelRightClickMenu_AddToggleIcon(ID)
	TitanPanelRightClickMenu_AddToggleLabelText(ID)
	TitanPanelRightClickMenu_AddToggleVar("Show Item Name", ID, "ShowItemName", nil, level)
	TitanPanelRightClickMenu_AddSpacer()
	TitanPanelRightClickMenu_AddCommand("Hide", ID, TITAN_PANEL_MENU_FUNC_HIDE, level)
end

--[[
-- ------------------[ Slash Command Handling ]---------------
]]--

SLASH_ITMCOUNT1 = "/tic";
function SlashCmdList.ITMCOUNT(msg)
   -- try getting count of item
   local icount = GetItemCount(msg)
   if icount > 0 then
      TitanItemCount.db.char.ItemToCount = msg
      TitanItemCount:ticAnnounce(msg)
      startVal = 0
   else
      self:Print(YELLOW_FONT_COLOR_CODE .. 
         " SYNTAX: |cff33DDFF/tic Item Name" .. 
         YELLOW_FONT_COLOR_CODE ..
         " (The Item must exist in your inventory)");
   end
end

